var express = require("express");
var bodyParser = require("body-parser");

var app = express();

app.use(bodyParser.urlencoded({ extended: false } ));
app.use(express.static('static'));

var server = app.listen(3000, function () {
	console.log('listening on port 3000');
})

var io = require("socket.io")(server);
var Myo = require("myo");

Myo.connect();
//var myMyo = Myo.create();
Myo.on("connected", function(data, timestamp) {
    console.log("Myo successfully connected. Data: " + JSON.stringify(data) + ". Timestamp: " + timestamp + ".");
	Myo.setLockingPolicy("none");
    //console.log(Myo);
});

var punchTime = 0;  
var onTestPunch = false;
var onTestRoll = "none";
var onTestPitch = "none";
var onTestYaw = "none";
var firstOrientationAngles;
var orientationRedo = true;

Myo.on("pose_off", function(poseName){
    var lastPoseName = poseName;
    if(poseName !== "double_tap" && poseName !== "fingers_spread"){
        if(onTestPunch){
            lastPoseName = poseName + ' punch';
            onTestPunch = false;
        }else{
            onTestPunch = false;
            if(onTestPitch === "up"){
                lastPoseName = poseName + ' up';
                //onTestPitch = "none";
            }else if(onTestPitch === "down"){
                lastPoseName = poseName + ' down';
                //onTestPitch = "none";
            }
            if(onTestRoll === "anticlockwise" && onTestPitch === "none"){
                lastPoseName = poseName + ' anti-clockwise';
                //onTestRoll = "none";
            }else if(onTestRoll === "clockwise" && onTestPitch === "none"){
                lastPoseName = poseName + ' clockwise';
                //onTestRoll = "none";
            }
            if(onTestYaw === "left" && onTestPitch === "none" && onTestRoll === "none"){
                lastPoseName = poseName + ' left';
                //onTestYaw = "none";
            }else if(onTestYaw === "right" && onTestPitch === "none" && onTestRoll === "none"){
                lastPoseName = poseName + ' right';
                //onTestYaw = "none";
            }
        }
        
        io.emit("logSession", lastPoseName);
        io.in("logging").emit('instantLog', lastPoseName);
    }else{
        io.emit("logSession", poseName);
        io.in("logging").emit('instantLog', poseName);
    }
    //orientationRedo = true;
});

Myo.on('imu', function(data) {  
    var time = (new Date()).getTime();
    if (punchTime < time - 1000 && data.accelerometer.x < -2.0) {
        punchTime = time;
        onTestPunch = true;
    }
});

Myo.on('orientation', function(data){
    if(orientationRedo){
	   getFirstOrientation();
    }
	var orientationAngles ={
		roll : getRoll(data),
		pitch : getPitch(data),
		yaw : getYaw(data)
	};
	//console.log(firstOrientationAngles);
    //console.log(orientationAngles);
    
    if(orientationAngles.roll > 9.5){
        onTestRoll = "anticlockwise";
    }else if(orientationAngles.roll < 5){
        onTestRoll = "clockwise";
    }else{
        onTestRoll = "none";
    }
    
    if(orientationAngles.pitch > 15){
        onTestPitch = "up";
    }else if(orientationAngles.pitch < 2){
        onTestPitch = "down";
    }else{
        onTestPitch = "none";
    }
    
    if(firstOrientationAngles.yaw > 4.5 && firstOrientationAngles.yaw < 13.5){
        var angleChanged = orientationAngles.yaw - firstOrientationAngles.yaw;
        if(angleChanged > 2){
            onTestYaw = 'left';
        }else if(angleChanged < -2){
            onTestYaw = 'right';
        }else{
            onTestYaw = 'none';
        }
    }else if(firstOrientationAngles.yaw < 4.5){
        var angleChanged = orientationAngles.yaw - firstOrientationAngles.yaw;
        if(angleChanged > 2 && angleChanged < 4.5){
            onTestYaw = 'left';
        }else if(angleChanged < -2 || angleChanged > 13.5){
            onTestYaw = 'right';
        }else{
            onTestYaw = 'none';
        }
    }else if(firstOrientationAngles.yaw > 13.5){
        var angleChanged = orientationAngles.yaw - firstOrientationAngles.yaw;
        if(angleChanged < -2 && angleChanged > -4.5){
            onTestYaw = 'right';
        }else if(angleChanged > 2 || angleChanged < -13.5){
            onTestYaw = 'left';
        }else{
            onTestYaw = 'none';
        }
    }
    
    console.log('roll: ' + onTestRoll);
    console.log('pitch: ' + onTestPitch);
    console.log('yaw: ' + onTestYaw);
    
});

function getFirstOrientation(){
	firstOrientationAngles ={
		roll : getRoll(Myo.myos[0].lastIMU.orientation),
		pitch : getPitch(Myo.myos[0].lastIMU.orientation),
		yaw : getYaw(Myo.myos[0].lastIMU.orientation)
	};
	for (var i = 0; i < 100; i++) {
		firstOrientationAngles.roll  += getRoll(Myo.myos[0].lastIMU.orientation);
		firstOrientationAngles.pitch += getPitch(Myo.myos[0].lastIMU.orientation);
		firstOrientationAngles.yaw   += getYaw(Myo.myos[0].lastIMU.orientation); 
	};

	firstOrientationAngles.roll /= 100;
	firstOrientationAngles.pitch /= 100;
	firstOrientationAngles.yaw /= 100;

    orientationRedo = false;
	//getFirstOrientation = noop;
}

//function noop(){}

function getRoll(data){
	var roll = Math.atan2(2.0 * (data.w * data.x + data.y * data.z), 1.0 - 2.0 * (data.x * data.x + data.y * data.y));
	var roll_w = ((roll + Math.PI)/(Math.PI * 2.0) * 18);
	return roll_w;
}

function getPitch(data){
	var pitch = Math.asin(Math.max(-1.0, Math.min(1.0, 2.0 * (data.w * data.y - data.z * data.x))));
	var pitch_w = ((pitch + Math.PI/2.0)/Math.PI * 18);
	return pitch_w;
}

function getYaw(data){
	var yaw = Math.atan2(2.0 * (data.w * data.z + data.x * data.y), 1.0 - 2.0 * (data.y * data.y + data.z * data.z));
	var yaw_w = ((yaw + Math.PI)/(Math.PI*2.0) * 18);
	return yaw_w;
}

var temp = {};

io.on('connection', function(socket){
  console.log('a user connected');

  socket.on('statusReq', function(){
  	console.log('user asked for status');
  	temp.info = "Response for status";
  	var data = Myo.myos[0];
  	//console.log(data);
    Myo.trigger("zeroOrientation");

  	socket.emit('status', temp, data);
  });

  socket.on('calibrationReq', function(){
  	console.log('user asked for calibration');
  	temp.info = "Response for calibration";

  	socket.emit('calibration', temp);
  });

  socket.on('logReq', function(roomData){
  	console.log('user asked for log');
  	temp.info = "Response for log";

  	socket.emit('log', temp);
  	socket.join(roomData.room);
  });

  socket.on('pairingReq', function(){
  	console.log('user asked for pairing');
  	temp.info = "Response for pairing";

  	socket.emit('pairing', temp);
  });

  socket.on('leaveLogging', function(){
  	socket.leave("logging");
  });

  socket.on('disconnect', function(){
    console.log('user disconnected');
  });
});

app.get('/', function(req, res){
	res.sendFile(__dirname + '/views/index.html');
});
